# This script is evaluated when it is needed to create a Logiscope
# project from an Eclipse project.

# Name: create
# Parameter: $obj the java selected object from Eclipse platform
#	     $tool the Logiscope tool for which the project should 
#	     be created (rule, audit, review, test)
# Return: None
#
# This proc is in charge of finding the IProject corresponding to the object
# selected in the Eclipse platform
# Once the project has been found, its language is searched.
# Then the list of files included into the project is built.
# Then the create proc corresponding to the $tool is called.
proc create {selection tool} {
    # Tries to find an Eclipse IProject from the Eclipse selection
    set project [eclog::getIProject $selection]

    # If the IProject is not found, just returns
    if {[eclog::isNull project]} {return {}}
    
    # Inform that the project has been found
    eclog::setProject $project
 
    # Get the language of the object $project (can be C, C++, Java or Ada)
    set language [eclog::getLanguage $project]

    # Depending on the language keep only certain kinds of files
    set files2Keep {}
    switch -- $language {
        java {
	    	set files2Keep $::JavaSources
        }
        c++ {
	    	set files2Keep $::CxxSources
	    	
	    	# Informations provided by the user through the GUI
	    	eclog::addProperty dialect [eclog::getDialect]
	    	eclog::addProperty macroDefinitions [eclog::getMacroDefinitions]

			# Extracts information from the CDT ICProject
    		uplevel #0 [list source [file join $::EclogDirectory cdt_script.tcl]]
	    	cdt_script::getCParams $project
	    	
	    	eclog::addProperty includesList $cdt_script::includesList
	    	eclog::addProperty CDTMacrosString $cdt_script::macrosString
	    	eclog::addProperty sourcesString $cdt_script::sourcesString	    
        }
        c {
	    	set files2Keep $::CSources

	    	# Informations provided by the user through the GUI
	    	eclog::addProperty dialect [eclog::getDialect]
	    	eclog::addProperty macroAndInclude [eclog::getMacroAndInclude]	 

		    set defFile [eclog::getDefFile]
		    if {[string length $defFile]} {
				eclog::addProperty defOpt "-def"
				eclog::addProperty defFile "$defFile"
			}		

		    set ignFile [eclog::getIgnFile]
		    if {[string length $ignFile]} {
				eclog::addProperty ignOpt "-ign"
				eclog::addProperty ignFile "$ignFile"
			}
			
			# Extracts information from the CDT ICProject
    		uplevel #0 [list source [file join $::EclogDirectory cdt_script.tcl]]
	    	cdt_script::getCParams $project

	    	eclog::addProperty includesList $cdt_script::includesList
	    	eclog::addProperty CDTMacrosString $cdt_script::macrosString
        }
    }
    
    # Set properties which are used in the ant build script
        
    # Set the repository for the project 
    eclog::addProperty repository [eclog::getRepository]
    
    # Set the pathname of the project
    eclog::addProperty ttp [eclog::getTtpPath $tool]

    # set the installation directory of Logiscope
    eclog::addProperty log_dir [file join [eclog::getInstallDirectory] bin]
    
    # set the current project directory
    eclog::addProperty prj_dir [eclog::getDirectory]
    
    # Initialize file extensions to nothing they will be filled by buildFileList
    set ::SourceExtensions {}
    set ::HeaderExtensions {}
    	
    # Get a temporary filename for the file list
    set temp [eclog::getTemporaryFile "logiscope" ".lst"]
    set f [open $temp w]
	buildFileList $f [$project members] $files2Keep
	close $f
    
    # Adds the temp filename as a property (named "tmp") for ant build
    eclog::addProperty tmp $temp

    # Now go to specific scripts
    set script [file join $::EclogDirectory create_${tool}.tcl]
    uplevel #0 [list source $script]

    # Call the create dedicated to the current tool    
    create_$tool $project $language
}

# Name: buildFileList
# Parameter: $out an opened file descriptor in which files names are written
#	     $members a Java array containing IResource objects
#            $files2keep list source file kinds to keep
# Return: None
#
# This proc finds the files to be included into the LOGISCOPE project.
# Each time an IFile of correct kind is found, its pathname is written
# in the $out file.
# This proc is recursive when an IResource is of kind IFolder.
proc buildFileList {out members files2Keep} {
    foreach elt [$members getrange] {
		if  {[$elt getType] == $::FILE_FIELD } {
		    if {[catch {[[[java::cast $::IFILE $elt] getContentDescription] getContentType] getId} res]} {
		    	# Can not get correct information for this file
				continue
		    }
		    if {[lsearch -exact $files2Keep $res] != -1} {
		        set path [[$elt getLocation] toString]
				puts $out $path
				eclog::addFile [java::cast $::IFILE $elt]
				set ext [file extension $path]
				if {[string match *Source $res]} {
					# a source file
					if {[lsearch -exact $::SourceExtensions $ext] == -1} {
						lappend ::SourceExtensions $ext
					}
				} elseif {[string match *Header $res]} {
					# a header file
					if {[lsearch -exact $::HeaderExtensions $ext] == -1} {
						lappend ::HeaderExtensions $ext
					}
				}
		    }
		} elseif {[$elt getType] == $::FOLDER_FIELD } {
		    buildFileList $out [[java::cast $::IFOLDER $elt] members] $files2Keep
		} 
    }    
}
