set EclogDirectory [file dirname [info script]]
source [file join $::EclogDirectory init.tcl]

namespace eval eclog {

variable LogProject {}

# Name: getIProject
# Parameter: $obj a java object from which we have to guess the corresponding IProject
# 
# Return: The java IProject found if any, {} otherwise
#
# This proc starting from $obj tries to go back to the corresponding IProject
proc getIProject {obj} {
    java::try {
        set project [[java::cast org.eclipse.core.resources.IResource $obj] getProject]
    } catch {TclException err} {
	java::try {
	    # was not an IResource, try the getResource method
	    set resource [$obj getResource]
	    set project [$resource getProject]
	} catch {TclException err} {
            set project {}
	}
    }
    return $project
}

# Name: getLanguage
# Parameter: $project an IProject java object
# 
# Return: The string representing the programming language of
# the project $project
#
# Depending on the nature associated to the project, this
# proc computes the language of the project.
proc getLanguage {project} {
    set isJavaProject [$project hasNature $::JAVA_NATURE]
    set isCppProject [$project hasNature $::CPP_NATURE]
    set isCProject [expr [$project hasNature $::C_NATURE] && !$isCppProject]

    set language "unknown"
    if {$isJavaProject && [extensionPresent $::JAVA_NATURE]} {
	set language java
    } elseif {$isCppProject && [extensionPresent $::CPP_NATURE]} {
	set language c++
    } elseif {$isCProject && [extensionPresent $::C_NATURE]} {
	set language c
    }
    return $language
}

# Name: getLanguages
# Parameter: None
# 
# Return: The list of languages currently available in Eclipse
#
# This proc guesses available languages by examining avaliable Java classes
# from package jdt and cdt
proc getLanguages {} {
    set lst {}
    
    # Searches for Java environement
    if {[extensionPresent $::JAVA_NATURE]} {
        lappend lst Java
    }
    # Searches for C++ environement
    if {[extensionPresent $::CPP_NATURE]} {
        lappend lst C++
    }
    # Searches for C environement
    if {[extensionPresent $::C_NATURE]} {
        lappend lst C
    }
    return $lst
}

proc extensionPresent {ext} {
    # Gets the extension registry from the plateform
    set regext [java::call org.eclipse.core.runtime.Platform getExtensionRegistry]
    if {[isNull regext]} {
        return 0
    }

    # Searches for the required extension
    set ext [$regext getExtension $ext]
    if {![isNull ext]} {
        return 1
    }
    return 0
}

proc setProject {project} {
    variable LogProject
    assert LogProject
    $LogProject setProject $project
}

proc addFile {file} {
    variable LogProject
    assert LogProject
    $LogProject addFile $file
}

proc addProperty {name value} {
    variable LogProject
    assert LogProject
    $LogProject addProperty $name $value
}

proc addViolation {lang fn ln start length mnemo msg} {
    variable LogProject
    assert LogProject

    set iFile [$LogProject {getFile String} $fn]
    if {![java::isnull $iFile]} {
        # We get the correct IFile object
        set mk [[java::cast org.eclipse.core.resources.IResource $iFile] createMarker $::ViolationMarker]
        if {[$mk exists]} {
            set MNEMO [java::field com.kalimetrix.logiscope.utilities.LogiscopeUtils MNEMONIC]
            set LANGUAGE [java::field com.kalimetrix.logiscope.utilities.LogiscopeUtils LANGUAGE]
            set level [getRuleState $mnemo]
            set mkClass org.eclipse.core.resources.IMarker
	   	 	$mk setAttribute [java::field $mkClass MESSAGE]  "$mnemo: $msg"
	    	$mk setAttribute $MNEMO  "$mnemo"
	    	$mk setAttribute $LANGUAGE  "$lang"
	    	$mk {setAttribute java.lang.String int} [java::field $mkClass PRIORITY] [java::field $mkClass PRIORITY_NORMAL]
	    	if {$level == 2} {
	    		$mk {setAttribute java.lang.String int} [java::field $mkClass SEVERITY] [java::field $mkClass SEVERITY_ERROR]
    	    } else {
	    		$mk {setAttribute java.lang.String int} [java::field $mkClass SEVERITY] [java::field $mkClass SEVERITY_WARNING]
    	    }
    	    if {![string equal $lang "c"]} {
    	    	if {[string equal $::tcl_platform(platform) "windows"]} {
    	    		set char_start [expr $start + $ln -1]
    	    		set char_end  [expr $start + $ln + $length -1]
	    		} else {
    	    		set char_start $start
    	    		set char_end  [expr $start + $length]
	    		}
	    		$mk {setAttribute java.lang.String int} [java::field $mkClass CHAR_START] $char_start
	    		$mk {setAttribute java.lang.String int} [java::field $mkClass CHAR_END] $char_end
    		}
	    	$mk {setAttribute java.lang.String int} [java::field $mkClass LINE_NUMBER] $ln
        }
    }
}

proc deleteViolations {} {
    variable LogProject
    assert LogProject
    
    set iProject [$LogProject getProject]
    set depth [java::field org.eclipse.core.resources.IResource DEPTH_INFINITE]
    $iProject deleteMarkers $::ViolationMarker true $depth
}

proc getTemporaryFile {prefix suffix} {
    set tmp [java::call java.io.File createTempFile $prefix $suffix]
    $tmp deleteOnExit
    return [$tmp getAbsolutePath]
}

proc isNull {name} {
    upvar $name obj
    set copy [format %s $obj]
    if {![string length $copy] || [java::isnull $copy]} {
        return 1
    }
    return 0
}

proc assert {name} {
    upvar $name obj
    set copy [format %s $obj]
    if {![string length $copy] || [java::isnull $copy]} {
        output "$name is null, but should not."
    }
}

proc getBooleanValue {name} {
    variable LogProject
    assert LogProject
    set valueStore [$LogProject getValueStore]
    set result [$valueStore getBooleanValue [java::field com.kalimetrix.logiscope.utilities.LogiscopeUtils $name]]
    return $result
}

proc getStringValue {name} {
    variable LogProject
    assert LogProject
    set valueStore [$LogProject getValueStore]
    set result [$valueStore getStringValue [java::field com.kalimetrix.logiscope.utilities.LogiscopeUtils $name]]
    return $result
}

proc getRelaxationState {} {
    return [getBooleanValue RELAX]
}

proc getFrameState {} {
    return [getBooleanValue FRAME]
}

proc getLinkedState {} {
    return [getBooleanValue LINKED]
}

proc getRuleState {mnemo} {
    variable LogProject
    assert LogProject
    set valueStore [$LogProject getValueStore]
    set result [$valueStore getIntValue "[java::field com.kalimetrix.logiscope.utilities.LogiscopeUtils RULE_STATE]$mnemo"]
    return $result
}

proc getRuleSets {} {
    variable LogProject
    assert LogProject
    set CURRENT [java::field com.kalimetrix.logiscope.utilities.LogiscopeUtils CURRENT_RULESETS]
    set valueStore [$LogProject getValueStore]
    set result [$valueStore getStringListValue $CURRENT]
    $valueStore setTclValue $CURRENT $result
    set lst {}
    java::for {String elem} $result {
       lappend lst $elem
    }    
    return $lst
}

proc getDialect {} {
    return [getStringValue DIALECT]
}

proc getMacroDefinitions {} {
    return [getStringValue DEFMAC]
}

proc getMacroAndInclude {} {
    return [getStringValue DEFINC]
}

proc getDefFile {} {
    return [getStringValue DEFINITION]
}

proc getIgnFile {} {
    return [getStringValue IGNORE]
}

proc getDirectory {} {
    variable LogProject
    assert LogProject
    set dir [$LogProject getDirectory]
    return $dir
}

proc getTtpDirectory {} {
    variable LogProject
    assert LogProject
    set valueStore [$LogProject getValueStore]
    set result [$valueStore getStringValue [java::field com.kalimetrix.logiscope.utilities.LogiscopeUtils DIRECTORY_NAME]]
    set result [file join [getDirectory] $result]
    return [file nativename $result]
}

proc getRepository {} {
    return [file join [getTtpDirectory] logiscope]
}

proc getReportDirectory {} {
    return [file join [getTtpDirectory] report]
}

proc getTtpPath {tool} {
    variable LogProject
    assert LogProject
    set iprj [$LogProject getProject]
    set result [$iprj getName]
    set result [file join [getTtpDirectory] ${result}_${tool}.ttp]
    return $result
}

proc getReportPath {tool} {
    variable LogProject
    assert LogProject
    set iprj [$LogProject getProject]
    set name [$iprj getName]
    set dir [getReportDirectory]
    set html [file join $dir ${name}_${tool}${tool}.html]
}

proc getInstallDirectory {} {
    set dir [java::call com.kalimetrix.logiscope.utilities.LogiscopeUtils getInstallDirectory]
    return [file nativename $dir]
}

proc makeFileLink {name fn} {
    variable LogProject
    assert LogProject
    set iprj [$LogProject getProject]
    set fl [$iprj getFile "$name"]
    if {![$fl exists]} {
        set path [java::new org.eclipse.core.runtime.Path $fn]
        $fl {createLink org.eclipse.core.runtime.IPath int org.eclipse.core.runtime.IProgressMonitor} \
                 $path [java::field org.eclipse.core.resources.IResource ALLOW_MISSING_LOCAL] [java::null]
    }
    return $fl
}

proc removeFileLink {name} {
    variable LogProject
    assert LogProject
    set iprj [$LogProject getProject]
    set fl [$iprj getFile "$name"]
    if {[$fl exists]} {
        $fl delete 1 0 [java::null]
    }
    return {}
}
}
